#ifndef PPM_IMAGE_H
#define PPM_IMAGE_H

#include <iostream>
#include <fstream>
#include <vector>
#include <string>

struct RGB
{
  int r{0};
  int g{0};
  int b{0};
  // RGB() : r(0), g(0), b(0) {}
};

class PPMImage
{
public:
  int width;
  int height;
  int maxColor;
  std::vector<std::vector<RGB>> data;

  PPMImage(int w, int h, int maxColorValue) : width(w), height(h), maxColor(maxColorValue)
  {
    data.resize(height, std::vector<RGB>(width));
  }

  // Read PPM image from a file
  bool readPPM(const std::string &filename)
  {
    std::ifstream file(filename, std::ios::in);

    if (!file)
    {
      std::cerr << "Unable to open the file " << filename << std::endl;
      return false;
    }

    std::string magic;
    file >> magic;
    if (magic != "P3")
    {
      std::cerr << "Invalid PPM file format." << std::endl;
      return false;
    }

    file >> width >> height >> maxColor;

    data.resize(height, std::vector<RGB>(width));

    for (int i = 0; i < height; ++i)
    {
      for (int j = 0; j < width; ++j)
      {
        file >> data[i][j].r >> data[i][j].g >> data[i][j].b;
      }
    }

    file.close();
    return true;
  }

  // Write PPM image to a file
  bool writePPM(const std::string &filename)
  {
    std::ofstream file(filename, std::ios::out);

    if (!file)
    {
      std::cerr << "Unable to create the file " << filename << std::endl;
      return false;
    }

    file << "P3" << std::endl;
    file << width << " " << height << std::endl;
    file << maxColor << std::endl;

    for (int i = 0; i < height; ++i)
    {
      for (int j = 0; j < width; ++j)
      {
        file << data[i][j].r << " " << data[i][j].g << " " << data[i][j].b << " ";
      }
      file << std::endl;
    }

    file.close();
    return true;
  }
};

#endif